package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.DepartmentService;
import com.bocloud.sms.model.DepartmentModel;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 组织管理控制器
 *
 * @author tyl
 * @since 2021/03/25
 */
@RestController
@RequestMapping("/v1/departments")
@Tag(name = "组织管理")
public class DepartmentController {

    private final DepartmentService departmentService;

    public DepartmentController(DepartmentService departmentService) {
        this.departmentService = departmentService;
    }

    /**
     * 查询组织机构
     *
     * @param parentId       父组织id
     * @param requestContext 上下文
     * @return 组织机构列表
     */
    @GetMapping
    @Operation(summary = "组织机构列表显示")
    public GeneralResult<List<Department>> list(@RequestParam(value = Common.PARENTID) Long parentId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.list(parentId, requestContext);
    }

    /**
     * 查询组织机构下的管理员
     *
     * @param id             组织id
     * @param requestContext 上下文
     * @return 管理员信息
     */
    @GetMapping("/{id}/managers")
    @Operation(summary = "查询组织机构下的管理员")
    public GeneralResult<List<User>> listDeptManager(@PathVariable(value = Common.ID) Long id, RequestContext requestContext) {
        return departmentService.listDeptManager(id, requestContext);
    }

    /**
     * 创建组织机构
     *
     * @param department     组织
     * @param requestContext 上下文
     * @return 创建结果
     */
    @Operation(summary = "创建组织机构")
    @PostMapping
    public Result create(@RequestBody DepartmentModel department,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.create(department, requestContext);
    }

    /**
     * 修改组织机构
     *
     * @param id             组织id
     * @param department     组织
     * @param requestContext 上下文
     * @return 修改结果
     */
    @Operation(summary = "修改组织机构")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody DepartmentModel department,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.modify(id, department, requestContext);
    }

    /**
     * 删除组织机构
     *
     * @param id             组织id
     * @param requestContext 上下文
     * @return 删除结果
     */
    @Operation(summary = "删除组织机构")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.remove(id, requestContext);
    }

    /**
     * 查询组织机构详情
     *
     * @param id 组织id
     * @return 组织
     */
    @Operation(summary = "组织机构详情")
    @GetMapping("/{id}")
    public GeneralResult<Department> detail(@PathVariable(value = Common.ID) Long id) {
        return departmentService.detail(id);
    }

}
