package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.booter.websocket.model.LogViewMessage;
import com.google.common.collect.Maps;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import com.megatron.common.utils.MapTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 当日日志和历史日志下载
 *
 * @author zhangdi
 * @since 2018.7.23
 */
@Slf4j
@RestController
@RequestMapping("/v1/logs")
@Tag(name = "日志下载管理")
public class LogDownloadController {

    @Value("${logging.dir:/log/services}")
    private String loggingDir;

    /**
     * 实时日志下载
     *
     * @param token    token
     * @param server   服务
     * @param host     host
     * @param request  浏览器请求
     * @param response 浏览器响应
     */
    @Operation(summary = "当日日志下载")
    @RequestMapping(value = "/download", method = {RequestMethod.GET})
    public void download(@RequestParam(value = "token") String token,
                         @RequestParam(value = "server") String server,
                         @RequestParam(value = "host") String host,
                         HttpServletRequest request, HttpServletResponse response) {
        Long position = 0L;
        Long presize = Long.MAX_VALUE;
        String fileName = server + ".log";
        String filepath = loggingDir + File.separator + fileName;
        LogViewMessage logViewMessage;
        switch (server) {
            case "worker": {
                String workPath = "/var/log/bocloud/bocloud_worker.log";
                String url = "http://" + host + "/api/" + server + "/v1/logview";
                logViewMessage = getRemoteLog(url, workPath, position, presize, host.substring(0, host.indexOf(':')));
                break;
            }
            case "pontus": {
                String workPath = "/var/log/pontus/pontus.log";
                String url = "http://" + host + "/api/" + server + "/v1/logview";
                logViewMessage = getRemoteLog(url, workPath, position, presize, null);
                break;
            }
            default: {
                String url = "http://" + host + "/v1/logview";
                log.info("token:{}", token);
                logViewMessage = getRemoteLog(token, url, filepath, position, presize);
                break;
            }
        }
        String content;
        try {
            content = JSONObject.toJSONString(logViewMessage.getContent());
            // 获取当前时间并拼成yyyy-MM-dd格式 HH:mm
            Date date = new Date();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String beginIndex = dateFormat.format(date).substring(0, 11) + "00:00";
            String endIndex = dateFormat.format(date);

            // 将开始结束时间放入日历类 操作日历截取日志
            Calendar beginTime = Calendar.getInstance();
            Calendar endTime = Calendar.getInstance();
            beginTime.setTime(dateFormat.parse(beginIndex));
            // beginTime.add(Calendar.MINUTE, -1);
            endTime.setTime(dateFormat.parse(endIndex));

            // 根据开始时间和结束时间进行索引
            int begin = content.indexOf(beginIndex);
            int end = content.indexOf(endIndex);
            // 判断开始时间是否存在，不存在则向后推一分钟
            int i = 0;
            while (begin == -1) {
                beginTime.add(Calendar.MINUTE, 1);
                SimpleDateFormat beginFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String newBeginIndex = beginFormat.format(beginTime.getTime());
                begin = content.indexOf(newBeginIndex);
                i++;
                if (i > 1440) {
                    break;
                }
            }

            // 判断结束时间是否存在，不存在则向前推一分钟
            int j = 0;
            while (end == -1) {
                endTime.add(Calendar.MINUTE, -1);
                SimpleDateFormat endFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String newEndIndex = endFormat.format(endTime.getTime());
                // 索引最后一次出现的结束时间
                end = content.lastIndexOf(newEndIndex);
                j++;
                if (j > 1440) {
                    break;
                }
            }

            // 判断此时间段内是否有日志
            String total;
            if (begin > end || begin == -1) {
                total = "此时间段内无日志";
            } else {
                String substring = content.substring(begin, end);
                // 补上最后时刻的一条数据
                String character = "\\n";
                // 从end的索引开始，向后拿到到“\n”为址的第一条数据
                int endAdd = content.indexOf(character, end);
                String add = content.substring(end, endAdd);
                total = substring + add;
                // 将捕获日志时拿到的“\n”、“\\”及着色符替换掉
                total = total.replace("\\\\", "").replace("\\n", "\r\n").replace("\\u001B[36m", "")
                        .replace("\\u001B[34m", "").replace("\\u001B[31m", "").replace("\\u001B[1;31m", "")
                        .replace("\\u001B[0;39m", "").replace("\\tat", "            ").replace("\\t...", "...");
            }
            response.setContentType("text/plain");
            String filename = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
            final String userAgent = request.getHeader("USER-AGENT");
            if (userAgent.contains("Mozilla")) {
                filename = new String(fileName.getBytes(), "ISO8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
            // 定义输出流
            OutputStream outputStream = response.getOutputStream();
            byte[] decode = total.getBytes();
            outputStream.write(decode);
            outputStream.close();
        } catch (Exception e) {
            log.error("Daily log download failed!");
        }

    }

    /**
     * 历史日志打包下载
     *
     * @param server   服务名称
     * @param host     host
     * @param begin    开始时间
     * @param end      结束时间
     * @param request  浏览器请求
     * @param response 浏览器响应
     */
    @Operation(summary = "历史日志下载")
    @RequestMapping(value = "/zip", method = {RequestMethod.GET})
    public void zip(@RequestParam(value = "token") String token,
                    @RequestParam(value = "server") String server,
                    @RequestParam(value = "host") String host,
                    @RequestParam(value = "begin") String begin,
                    @RequestParam(value = "end") String end,
                    HttpServletRequest request, HttpServletResponse response) {
        try {
            String filepath = loggingDir + File.separator + "history";
            // 定义导出文件名称
            String fileName = server + ".zip";
            String url = "http://" + host + "/v1/logzip";
            Map<String, byte[]> map = getRemoteZip(token, url, filepath, begin, end, server);

            response.setContentType("application/x-zip-compressed");
            String filename = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
            final String userAgent = request.getHeader("USER-AGENT");
            if (userAgent.contains("Mozilla")) {
                filename = new String(fileName.getBytes(), "ISO8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
            // 创建一个临时文件
            File file = new File(System.getProperty("user.home") + File.separator + fileName);
            OutputStream outputStream = response.getOutputStream();
            try (ZipOutputStream zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
                byte[] buffer = new byte[1024];
                // 遍历拿到的每个文件的数据
                for (Map.Entry<String, byte[]> entry : map.entrySet()) {
                    InputStream inputStream = new ByteArrayInputStream(entry.getValue());
                    // 文件命名
                    zipOutputStream.putNextEntry(new ZipEntry(entry.getKey()));
                    int len;
                    // 读入需要下载的文件的内容
                    while ((len = inputStream.read(buffer)) > 0) {
                        zipOutputStream.write(buffer, 0, len);
                    }
                    zipOutputStream.closeEntry();
                }
            } catch (IOException e) {
                log.error("Get error message：", e);
            }

            try (FileInputStream fileInputStream = new FileInputStream(file)) {
                IOUtils.copy(fileInputStream, outputStream);
                outputStream.close();
            } catch (IOException e) {
                log.error("Get error message：", e);
            }
            // 删除临时文件
            boolean flag = file.delete();
            log.info("临时文件删除状态：{}", flag);
        } catch (Exception e) {
            log.error("Get error message：", e);
        }
    }

    /**
     * 获取远端服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @return
     */
    private LogViewMessage getRemoteLog(String token, String url, String filepath, Long position, Long presize) {
        LogViewMessage message;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        Result result = client.get(MapTools.simpleMap("token", token), param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null != jsonObject) {
                jsonObject = JSONObject.parseObject(jsonObject.getString("message"));
            } else {
                jsonObject = new JSONObject();
            }
            message = new LogViewMessage(jsonObject.getString("content"), jsonObject.getLong("position"));
        }
        return message;
    }

    /**
     * 获取远端worker服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @param target   worker服务器IP地址
     * @return
     */
    private LogViewMessage getRemoteLog(String url, String filepath, Long position, Long presize, String target) {
        LogViewMessage message;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        if (StringUtils.isNotEmpty(target)) {
            param.put("target", target);
        }
        Result result = client.post(null, param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            message = new LogViewMessage(jsonObject.getString("message"), jsonObject.getLong("filesize"));
        }
        return message;
    }


    /**
     * 获取远程服务器历史日志
     *
     * @param url      远程地址
     * @param filepath 文件路径
     * @param begin    开始实际按
     * @param end      结束实际按
     * @param server   服务名称
     * @return 历史日志byte
     */
    private Map<String, byte[]> getRemoteZip(String token, String url, String filepath, String begin, String end,
                                             String server) {
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("begin", begin);
        param.put("end", end);
        param.put("server", server);
        Result result = client.get(MapTools.simpleMap("token", token), param, url);

        Map<String, byte[]> dataMap = Maps.newHashMap();
        if (result.isSuccess() && StringUtils.isNotEmpty(result.getMessage())) {
            JSONObject object = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            log.info("获取到日志结果为 {} ", JSONObject.toJSONString(object));
            for (String key : object.keySet()) {
                try {
                    dataMap.put(key, object.getBytes(key));
                } catch (Exception e) {
                    log.error("获取日志异常 {} , ->>>>>{}", JSONObject.toJSONString(key), object.get(key));
                }
            }
        }
        return dataMap;
    }
}
