package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.entity.Plugin;
import com.bocloud.sms.interfaces.PluginService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 插件管理
 *
 * @author zhangyf
 * @since 2022-05-20
 */
@Slf4j
@RestController
@RequestMapping("/v2/plugins")
@Tag(name = "插件管理")
public class PluginController {

    private final PluginService pluginService;

    @Value("${spring.upload.path:/home/cmp}")
    private String path;
    @Value("${plugin.repository.path:/home/cmp/plugins/repository/}")
    private String pluginRepoPath;

    public PluginController(PluginService pluginService) {
        this.pluginService = pluginService;
    }

    /*-------------------------------------------- 不可见插件列表接口 --------------------------------------------*/

    /**
     * 新增Plugin插件
     *
     * @param name       名称
     * @param vendorType 平台
     * @param service    主服务
     * @param filepath   插件包
     * @return 新增结果
     */
    @PostMapping
    @Operation(summary = "新增插件")
    public Result create(@RequestParam(value = "name") String name,
                         @RequestParam(value = "vendorType", required = false) String vendorType,
                         @RequestParam(value = "service", required = false) String service,
                         @RequestParam(value = "filepath", required = false) String filepath) throws IOException {
        File tmpFile = new File(path + File.separator + filepath);
        File destFile = new File(pluginRepoPath + File.separator + service, tmpFile.getName());
        try {
            if (!destFile.getParentFile().exists()) {
                if (!destFile.getParentFile().mkdirs()) {
                    log.error("mkdir {} failed}", destFile.getParentFile());
                }
            }
            FileUtils.copyFile(tmpFile, destFile);
            FileUtils.delete(tmpFile);
            if (FileUtils.isEmptyDirectory(tmpFile.getParentFile())) {
                FileUtils.deleteDirectory(tmpFile.getParentFile());
            }
            Plugin plugin = new Plugin();
            plugin.setService(service);
            plugin.setName(name);
            plugin.setService(service);
            plugin.setVendorType(vendorType);
            plugin.setSize(FileUtils.sizeOf(destFile));
            plugin.setPath(destFile.getName());
            return pluginService.create(plugin);
        } catch (RuntimeException | IOException e) {
            if (destFile.exists() && !destFile.delete()) {
                log.warn("临时文件删除失败：{}", destFile.getPath());
            }
            throw e;
        }
    }

    /**
     * 修改Plugin插件
     *
     * @param id         ID
     * @param name       名称
     * @param vendorType 平台类型
     * @param service    主服务
     * @param filepath   插件包
     * @return 修改结果
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改插件")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "name") String name,
                         @RequestParam(value = "vendorType", required = false) String vendorType,
                         @RequestParam(value = "service", required = false) String service,
                         @RequestParam(value = "filepath", required = false) String filepath) throws IOException {
        boolean hasPackage = filepath != null;
        File tmpFile = filepath != null ? new File(path + File.separator + filepath) : null;
        File destFile = null;
        if (hasPackage) {
            destFile = new File(pluginRepoPath + File.separator + service, tmpFile.getName());
        }
        try {
            if (hasPackage) {
                if (!destFile.getParentFile().exists()) {
                    if (!destFile.getParentFile().mkdirs()) {
                        log.error("mkdir {} failed}", destFile.getParentFile());
                    }
                }
                FileUtils.copyFile(tmpFile, destFile);
                FileUtils.delete(tmpFile);
                if (FileUtils.isEmptyDirectory(tmpFile.getParentFile())) {
                    FileUtils.deleteDirectory(tmpFile.getParentFile());
                }
            }
            Plugin plugin = new Plugin();
            plugin.setService(service);
            plugin.setName(name);
            plugin.setVendorType(vendorType);
            plugin.setSize(hasPackage ? FileUtils.sizeOf(destFile) : null);
            plugin.setPath(hasPackage ? destFile.getName() : null);
            return pluginService.modify(id, plugin, hasPackage);
        } catch (RuntimeException | IOException e) {
            if (hasPackage) {
                if (destFile.exists() && !destFile.delete()) {
                    log.warn("临时文件删除失败：{}", destFile.getPath());
                }
            }
            throw e;
        }
    }

    /**
     * 删除Plugin插件
     *
     * @param id 插件id
     * @return 删除结果
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除插件")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return pluginService.remove(id);
    }

    /**
     * 批量删除Plugin插件
     *
     * @param idList 插件id列表
     * @return 成功/失败 个数
     */
    @DeleteMapping("/batch")
    @Operation(summary = "批量删除插件")
    public Result batchRemove(@RequestBody List<Long> idList) {
        return pluginService.batchRemove(idList);
    }

    @GetMapping("/{service}/installed")
    @Operation(summary = "已安装插件列表")
    public GeneralResult<Map<String, List<Plugin>>> listInstalled(@PathVariable(value = "service") String service) {
        Map<String, List<Plugin>> pluginList = pluginService.listInstalled(service);
        return new GeneralResult<>(true, pluginList, "查询成功");
    }

    /**
     * 可更新插件列表
     *
     * @param service 服务name
     * @return 插件列表
     */
    @GetMapping("/{service}/update")
    @Operation(summary = "可更新插件列表")
    public GeneralResult<Map<String, List<Plugin>>> listUpdate(@PathVariable(value = "service") String service) {
        Map<String, List<Plugin>> pluginList = pluginService.listUpdate(service);
        return new GeneralResult<>(true, pluginList, "查询成功");
    }

    /**
     * 已安装插件列表
     *
     * @param service 服务名称
     * @return 插件列表
     */
    @GetMapping("/{service}/install")
    @Operation(summary = "已安装插件列表")
    public GeneralResult<Map<String, List<Plugin>>> listCanInstall(@PathVariable(value = "service") String service) {
        Map<String, List<Plugin>> pluginList = pluginService.listCanInstall(service);
        return new GeneralResult<>(true, pluginList, "查询成功");
    }

    /**
     * 插件列表
     *
     * @param pager 分页参数
     * @return 插件列表
     */
    @GetMapping
    @Operation(summary = "插件列表")
    public GeneralResult<GridBean<Plugin>> list(Pager pager) {
        return pluginService.list(pager);
    }

    /**
     * 插件列表
     *
     * @param code 分页参数
     * @return 插件列表
     */
    @GetMapping("/{code}/upgrade")
    @Operation(summary = "插件列表")
    public GeneralResult<GridBean<Plugin>> upgradeList(@PathVariable(value = "code") String code, Pager pager) {
        pager.getParams().add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        return pluginService.list(pager);
    }

    /**
     * 插件列表
     *
     * @param code 分页参数
     * @return 插件列表
     */
    @GetMapping("/{code}/degradation")
    @Operation(summary = "插件列表")
    public GeneralResult<GridBean<Plugin>> degradationList(@PathVariable(value = "code") String code, Pager pager) {
        pager.getParams().add(new Param(MapTools.simpleMap("code", code), Sign.EQ));
        return pluginService.list(pager);
    }

    /**
     * 查询插件详情
     *
     * @param id 插件id
     * @return 插件
     */
    @GetMapping("/{id}")
    @Operation(summary = "插件详情")
    public GeneralResult<Plugin> detail(@PathVariable(value = Common.ID) Long id) {
        return new GeneralResult<>(true, pluginService.detail(id), "查询成功");
    }

    /**
     * 获取插件支持的云平台列表信息（资源接入）
     *
     * @param pager 分页参数
     * @return 插件支持的云平台列表信息
     */
    @GetMapping("/vendors")
    @Operation(summary = "插件支持的云平台列表")
    public GeneralResult<List<Dictionary>> getVendorList(Pager pager) {
        return pluginService.getVendorList(pager);
    }

    /**
     * 获取license中的插件列表
     *
     * @param pager 分页参数
     * @return license中的插件列表
     */
    @GetMapping("/license")
    @Operation(summary = "获取Plugin插件列表")
    public GeneralResult<List<Plugin>> licenseList(Pager pager) {
        List<Plugin> plugins = pluginService.listLicense(pager);
        return new GeneralResult<>(true, plugins, "查询成功");
    }

}
