package com.bocloud.sms.booter.websocket.logview;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.booter.websocket.model.LogViewMessage;
import com.google.common.collect.Lists;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.LogBean;
import com.megatron.common.model.Result;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.LogCrawler;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.RegistryService;
import com.megatron.framework.core.Service;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.socket.*;

import java.io.File;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 11:21 2018/11/5 @ Modified By：
 */
@Slf4j
@Component
public class LogViewHandler implements WebSocketHandler {
    private static final String FETCH_SERVICE = "/logview";
    private static final String[] EXCLUDE_SERVERS =
            {"tds.server", "ams.server", "cms.server", "mcs.server", "cop.server", "tse.server", "gaia.server",
                    "cos.server", "sms.server", "gms.server", "ros.server", "ams.server", "bms.server", "cmp.server"};
    private final RegistryService registryService;
    @Value("${logging.dir:/log/services}")
    private String loggingDir;

    public LogViewHandler(RegistryService registryService) {
        this.registryService = registryService;
    }

    private String buildUrl(String host, String value) {
        return "http://" + host + (!StringUtils.hasText(value) ? "" : value) + FETCH_SERVICE;
    }

    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        if (message instanceof TextMessage textMessage) {
            String path = new File(loggingDir).getPath() + File.separator;
            JSONObject messageJson = JSONTools.isJSONObj(textMessage.getPayload());
            if (null == messageJson) {
                return;
            }
            String log = messageJson.getString("logger");
            String host = messageJson.getString("host");
            Long position = messageJson.getLong("position");
            Long presize = messageJson.getLong("logsize");
            if (null == position || position <= 0) {
                position = 0L;
            }
            if (null == presize) {
                presize = 0L;
            }
            LogViewMessage logViewMessage = null;
            if (Arrays.asList(EXCLUDE_SERVERS).contains(log)) {
                path += log.split("\\.")[0] + ".log";
                String url = buildUrl(host, "/v1");
                logViewMessage = getRemoteLog(url, path, position, presize);
            } else if ("gaia.server".equals(log)) {
                String url = buildUrl(host, "/api/" + log.split("\\.")[0] + "/v1");
                logViewMessage = getRemoteLog(url, "", position, presize, host.split(":")[0]);
            } else if ("pontus.server".equals(log)) {
                String url = buildUrl(host, "/api/" + log.split("\\.")[0] + "/v1");
                logViewMessage = getRemoteLog(url, "", position, presize, null);
            } else if (StringUtils.hasText(log) && log.contains("service.log")) {
                String user = messageJson.getString("user");
                String password = messageJson.getString("password");
                String target = messageJson.getString("target");
                path = messageJson.getString("filepath");
                String category = messageJson.getString("category");
                Integer port = messageJson.getInteger("port");
                logViewMessage = getRemoteLog(path, position, presize, user, password, port, target, category);
            }
            session.sendMessage(new TextMessage(JSONObject.toJSONString(logViewMessage)));
        } else {
            log.error("Unexpected WebSocket message type: " + message);
            throw new IllegalStateException("Unexpected WebSocket message type: " + message);
        }

    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        if (session.isOpen()) {
            session.close();
        }
    }

    @Override
    public void afterConnectionClosed(WebSocketSession webSocketSession, CloseStatus closeStatus) throws Exception {

    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    /**
     * 获取远端服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @return
     */
    private LogViewMessage getRemoteLog(String url, String filepath, Long position, Long presize) {
        LogViewMessage message = null;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        Result result = client.get(null, param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            jsonObject = JSONObject.parseObject(jsonObject.getString("message"));
            String originalMessage = jsonObject.getString("content");
            String msg = originalMessage.replaceAll("\\[(.{2,4})[m]", "");
            message = new LogViewMessage(msg, jsonObject.getLong("position"));
        }
        return message;
    }

    /**
     * 获取远端服务器上的日志文件内容
     *
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @param user     用户账号
     * @param password 用户密码
     * @param port     目标端口
     * @return
     */
    private LogViewMessage getRemoteLog(String filepath, Long position, Long presize, String user, String password,
                                        Integer port, String host, String category) {
        String address = registryService.getService(Service.create(BoCloudService.GAIA.getName()));
        if (!StringUtils.hasText(address)) {
            return new LogViewMessage("远端日志获取异常");
        }
        String url = address + "/api/gaia/v1/job/submit";
        log.info("服务地址:" + url);
        LogViewMessage viewMessage;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        // 同步请求
        Map<String, Object> param = MapTools.simpleMap("response", "sync");
        // module设置
        Map<String, Object> module = MapTools.simpleMap("name", "logview");
        Map<String, Object> logview = MapTools.simpleMap("position", position);
        logview.put("presize", presize);
        logview.put("log_file", filepath);
        Map<String, Object> args = MapTools.simpleMap("gaia_logview", logview);
        module.put("args", args);
        param.put("module", module);
        Map<String, Object> target = MapTools.simpleMap("host", host);
        target.put("pasd", password);
        target.put("user", user);

        if (!StringUtils.hasText(category) || !"windows".equalsIgnoreCase(category)) {
            category = "Linux";
        }
        target.put("category", category);
        if (null == port) {
            port = "windows".equalsIgnoreCase(category) ? 5985 : 22;
        }
        target.put("port", port);
        List<Object> targets = Lists.newArrayList();
        targets.add(target);
        param.put("targets", targets);
        Result result = client.post(null, param, url);
        log.info("请求结果内容:" + JSONObject.toJSONString(result));
        if (result.isFailed()) {
            viewMessage = new LogViewMessage(false, "文件可能不存在或文件为空", position);
        } else {
            JSONObject object = JSONObject.parseObject(result.getMessage());
            List<JSONObject> datas = JSONObject.parseArray(object.getString("data"), JSONObject.class);
            if (!ListTool.isEmpty(datas)) {
                JSONObject data = datas.get(0);
                JSONObject message = data.getJSONObject("message");
                if (null != object.getBoolean("success") && object.getBoolean("success")) {
                    String content = null;
                    if (null != message) {
                        content = message.getString("content");
                    }
                    viewMessage = new LogViewMessage(true, content, object.getLong("curr_position"));
                } else {
                    viewMessage = new LogViewMessage(false, "文件不存在或存在的文件无法打开", position);
                }
            } else {
                viewMessage = new LogViewMessage(false, "文件可能不存在或文件为空", position);
            }
        }
        return viewMessage;
    }

    /**
     * 获取远端worker服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @param target   worker服务器IP地址
     * @return
     */
    private LogViewMessage getRemoteLog(String url, String filepath, Long position, Long presize, String target) {
        LogViewMessage message;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        if (StringUtils.hasText(target)) {
            param.put("target", target);
        }
        Result result = client.post(null, param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            String originalMessage = jsonObject.getString("message");
            String msg = originalMessage.replaceAll("\\[(.{2,4})[m]", "");
            message = new LogViewMessage(msg, jsonObject.getLong("filesize"));
        }
        return message;
    }

    private LogViewMessage getLocalLog(String filepath, Long position, Long presize) {
        LogViewMessage message = null;
        GeneralResult<LogBean> crawlResult = LogCrawler.crawl(filepath, position, presize);
        if (crawlResult.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            LogBean logBean = crawlResult.getData();
            String originalMessage = logBean.getContent();
            String msg = originalMessage.replaceAll("\\[(.{2,4})[m]", "");
            message = new LogViewMessage(msg, logBean.getPosition());
        }
        return message;
    }
}