package com.bocloud.sms.interfaces;

import com.bocloud.sms.entity.*;
import com.bocloud.sms.model.*;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.*;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * 用户抽象Service接口
 *
 * @author tyl
 */
public interface UserService {

    /**
     * 用户登录
     *
     * @param account
     * @param password
     * @param sessionId
     * @param requestIp
     * @param isManager
     * @return
     * @throws InternalServerException
     */
    GeneralResult<Map<String, Object>> login(String account, String password, String sessionId, String requestIp, Boolean isManager) throws InternalServerException;

    /**
     * 用户退出
     *
     * @param id
     * @return
     * @throws InternalServerException
     */
    Result logout(Long id) throws InternalServerException;

    /**
     * 创建用户
     *
     * @param userBean
     * @param requestContext
     * @return
     */
    Result create(UserBean userBean, RequestContext requestContext);

    /**
     * 更新用户
     *
     * @param id
     * @param userBean
     * @param requestContext
     * @return
     */
    Result modify(Long id, UserBean userBean, RequestContext requestContext);

    /**
     * 删除用户
     *
     * @param id
     * @param requestContext
     * @return
     */
    Result remove(Long id, RequestContext requestContext);

    /**
     * 修改用户状态
     *
     * @param id
     * @param action         lock:锁定 active: 激活
     * @param requestContext
     * @return
     */
    Result updateStatus(Long id, ActionModel.Action action, RequestContext requestContext);

    /**
     * 批量冻结用户
     *
     * @param ids
     * @param requestContext
     * @return
     */
    Result batchLock(List<Long> ids, RequestContext requestContext);

    /**
     * 批量解冻用户
     *
     * @param ids
     * @param requestContext
     * @return
     */
    Result batchActive(List<Long> ids, RequestContext requestContext);

    /**
     * 批量删除用户
     *
     * @param ids
     * @param requestContext
     * @return
     */
    Result batchRemove(List<Long> ids, RequestContext requestContext);

    /**
     * 修改用户密码
     *
     * @param id
     * @param oldPassword
     * @param password
     * @param requestContext
     * @return
     */
    Result changePwd(Long id, String oldPassword, String password, RequestContext requestContext);

    /**
     * 重置密码
     *
     * @param id
     * @param password
     * @param requestContext
     * @return
     */
    Result resetPwd(Long id, String password, RequestContext requestContext);

    /**
     * 授权角色
     *
     * @param id
     * @param roleIds
     * @param requestContext
     * @return
     */
    Result accredit(Long id, List<Long> roleIds, RequestContext requestContext);

    /**
     * 查看用户详情
     *
     * @param id
     * @return
     */
    GeneralResult<User> detail(Long id);

    /**
     * 查询用户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    GeneralResult<GridBean<User>> list(Pager pager, RequestContext requestContext);

    /**
     * 获取安全密钥
     *
     * @param apiKey
     * @return
     */
    GeneralResult<AccountSecurity> getAccountSecurity(String apiKey);

    /**
     * 校验密码
     *
     * @param id
     * @param password
     * @return
     */
    Result check(Long id, String password);

    /**
     * 查询项目下用户
     *
     * @param projectId
     * @param requestContext
     * @return
     */
    GeneralResult<List<User>> listProjectUser(Long projectId, RequestContext requestContext);

    /**
     * 查询项目下项目经理
     *
     * @param id
     * @return
     */
    GeneralResult<List<User>> listProjectManager(Long id);

    /**
     * 检查用户是否异地登录
     *
     * @param id
     * @param token
     * @return
     */
    Result checkUser(Long id, String token);

    /**
     * 查询用户的角色
     *
     * @param id
     * @param requestContext
     * @return
     */
    GeneralResult<List<Role>> listRoles(Long id, String condition, RequestContext requestContext);


    /**
     * 根据账号冻结账号基本信息
     *
     * @param account
     * @return
     * @throws InternalServerException
     */
    Result lockByAccount(String account) throws InternalServerException;

    /**
     * 保存用户登录数据
     *
     * @param log
     * @return
     * @throws InternalServerException
     */
    Result save(LoginRecord log) throws InternalServerException;

    /**
     * 用户登录记录列表展示
     *
     * @param pager
     * @return
     */
    GeneralResult<GridBean<LoginRecord>> list(Pager pager);

    /**
     * 添加收藏
     *
     * @param serviceMenuId
     * @param requestContext
     * @return
     */
    Result addFavorite(Long serviceMenuId, RequestContext requestContext);

    /**
     * 取消收藏
     *
     * @param serviceMenId
     * @param requestContext
     * @return
     */
    Result cancelFavorite(Long serviceMenId, RequestContext requestContext);

    /**
     * 获取服务目录
     *
     * @param requestContext
     * @return
     */
    GeneralResult<List<ServiceMenuBean>> listServiceMenu(RequestContext requestContext);

    /**
     * 获取收藏夹
     *
     * @param requestContext
     * @return
     */
    GeneralResult<List<CloudServiceBean>> listFavorite(RequestContext requestContext);


    /**
     * 修改收藏夹排序
     *
     * @param params
     * @param requestContext
     * @return
     */
    Result changePriority(List<PriorityModel> params, RequestContext requestContext);

    /**
     * 返回账号相关登录所需信息
     *
     * @param account
     * @return
     */
    GeneralResult<Map<String, Object>> tokenLogin(String account);

    /**
     * 根据账号查询用户信息
     *
     * @param account
     * @return
     */
    Result exist(String account);

    /**
     * 查询用户项目
     *
     * @param id
     * @return
     */
    GeneralResult<List<Project>> projects(Long id);

    /**
     * 用户导入模板
     *
     * @param title
     * @param response
     * @return
     */
    Result importTemp(String title, HttpServletResponse response);

    /**
     * 用户数据导入
     *
     * @param multipartFile
     * @param tenantId
     * @param requestContext
     * @return
     */
    Result importData(MultipartFile multipartFile, Long tenantId, RequestContext requestContext);

    /**
     * 用户数据导出
     *
     * @param paramList
     * @param requestContext
     * @param response
     * @return
     */
    Result export(List<Param> paramList, RequestContext requestContext, HttpServletResponse response);

    /**
     * 用户移除所属租户
     *
     * @param id
     * @param requestContext
     * @return
     */
    Result unBindTenants(Long id, RequestContext requestContext);


    GeneralResult checkPassword(RequestContext requestContext, String password);


}
