package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.entity.Label;
import com.bocloud.sms.interfaces.LabelService;
import com.bocloud.sms.repository.LabelRepository;
import com.google.common.collect.Maps;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * @author tyl
 */
@Service("labelService")
@Slf4j
public class LabelServiceImpl implements LabelService {


    private final LabelRepository labelRepository;
    private final LockFactory lockFactory;

    public LabelServiceImpl(LabelRepository labelRepository, LockFactory lockFactory) {
        this.labelRepository = labelRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public GeneralResult<GridBean<Label>> list(Pager pager, RequestContext context) {
        List<Label> list;
        int total;
        GridBean<Label> gridBean;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            int page = Optional.ofNullable(pager.getPage()).orElse(1);
            int rows = Optional.ofNullable(pager.getRows()).orElse(10);
            Long target = 0L;
            if (RequestContext.Catalog.Tenant.equals(context.getCatalog())) {
                target = context.getTarget();
            } else if (RequestContext.Catalog.User.equals(context.getCatalog())) {
                target = context.getTenant();
            }
            total = labelRepository.count(params, target);
            if (simple) {
                List<Label> beans = this.labelRepository.list(params, target);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = labelRepository.list(page, rows, params, sorter, target);
                for (Label label : list) {
                    if (StringUtils.hasText(label.getContent())) {
                        List<String> values = JSONArray.parseArray(label.getContent(), String.class);
                        label.setValues(values);
                    }
                }
                gridBean = new GridBean<>(page, rows, total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list label failure:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Label> detail(Long id) {
        try {
            Label label = labelRepository.query(id);
            if (StringUtils.hasText(label.getContent())) {
                List<String> values = JSONArray.parseArray(label.getContent(), String.class);
                label.setValues(values);
            }
            return new GeneralResult<>(true, label, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = DataBaseException.class)
    public GeneralResult<Label> create(Label label, RequestContext context) {
        String path = Label.class.getSimpleName() + "_create";
        int labelContentMaxLength = 500;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(30, TimeUnit.SECONDS), "请求超时");
            if (labelRepository.exists(label.getName())) {
                return new GeneralResult<>(false, "标签已存在");
            }
            label.setContent(JSONArray.toJSONString(label.getValues()));
            if (label.getContent().length() > labelContentMaxLength) {
                return new GeneralResult<>(false, "标签内容长度不能超过500个字符");
            }
            label.setCreateId(context.getTarget());
            if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
                label.setTenantId(0L);
                label.setPersonal(false);
            } else {
                label.setTenantId(context.getTenant());
                label.setPersonal(true);
            }
            labelRepository.save(label);
            return new GeneralResult<>(true, label, "添加成功");
        } catch (Exception e) {
            log.error("Create label error:", e);
            return new GeneralResult<>(false, "添加失败");
        }
    }

    @Override
    public GeneralResult<Label> modify(Label label) {
        String path = Label.class.getSimpleName() + "_" + label.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Label oldLabel = this.labelRepository.query(label.getId());
            if (null == oldLabel) {
                log.warn("Label does not exist!");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!label.getName().equals(oldLabel.getName())) {
                if (labelRepository.exists(label.getName())) {
                    return new GeneralResult<>(false, "名称已存在");
                }
            }
            oldLabel.setName(label.getName());
            oldLabel.setRemark(label.getRemark());
            oldLabel.setContent(JSONArray.toJSONString(label.getValues()));
            this.labelRepository.update(oldLabel);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify label error:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public Result remove(Long id) {
        String path = Label.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Label oldLabel = this.labelRepository.query(id);
            if (null == oldLabel) {
                log.warn("Label does not exist!");
                return new GeneralResult<>(false, "数据不存在");
            }
            this.labelRepository.remove(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove label error:", e);
            return new Result(false, "删除失败");
        }
    }
}
