package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.entity.AccessLog;
import com.bocloud.sms.entity.LogArchiveRecord;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.LogService;
import com.bocloud.sms.model.log.LogModel;
import com.bocloud.sms.repository.LogArchiveRecordRepository;
import com.bocloud.sms.repository.LogRepository;
import com.bocloud.sms.repository.UserRepository;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.shaded.com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 审计日志服务实现类
 *
 * @author dmw
 */
@Service("logService")
@Slf4j
@RequiredArgsConstructor
public class LogServiceImpl implements LogService {

    private static SimpleDateFormat simpleFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private static final String REG_EXP = "(?:')|(?:--)|(#+)|(\\\\n)|(/\\*(?:.|[\\n\\r])*?\\*/)";
    private final StringRedisTemplate redisTemplate;
    private final LogRepository logRepository;
    private final LogArchiveRecordRepository logArchiveRecordRepository;
    private final UserRepository userRepository;
    @Autowired
    private KafkaTemplate<String, LogModel> kafkaTemplate;

    @Value("${log.push.systemNumber:0}")
    private String systemNumber; // 日志推送系统编号
    @Value("${log.push.systemNameCn:0}")
    private String systemNameCn; // 日志推送系统名称
    @Value("${log.push.topic:YK_USER_LOG_TOPIC}")
    private String topic; // 日志推送主题名称
    @Value("${log.push.enable:false}")
    private Boolean enable; // 日志推送开关
    @Value("${spring.cloud.zookeeper.discovery.instance-host:127.0.0.1}")
    private String host;

    @Override
    public GeneralResult<GridBean<AccessLog>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            int total = this.logRepository.count(params);
            List<AccessLog> list = new ArrayList<>();
            if (total > 0) {
                list = this.logRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            }
            for (AccessLog accessLog : list) {
                if (accessLog.getCatalog() != null) {
                    if (accessLog.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                        accessLog.setAccount(
                                redisTemplate.opsForValue().get(accessLog.getCatalog() + "_" + accessLog.getUserId()));
                        accessLog.setUserName(
                                redisTemplate.opsForValue().get(accessLog.getCatalog() + "_Name_" + accessLog.getUserId()));
                    } else {
                        accessLog.setAccount(redisTemplate.opsForValue()
                                .get(RequestContext.Catalog.User.name() + "_" + accessLog.getUserId()));
                        accessLog.setUserName(redisTemplate.opsForValue()
                                .get(RequestContext.Catalog.User.name() + "_Name_" + accessLog.getUserId()));
                    }
                }
            }
            GridBean<AccessLog> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query accessLog list fail:", e);
            return new GeneralResult<>(false, "查询失败", null, null);
        }
    }

    @Override
    public Result create(AccessLog accessLog) {
        try {
            this.logRepository.save(accessLog);
            return new Result(true, "添加成功");
        } catch (Exception e) {
            log.error("Create accessLog fail:", e);
            return new Result(false, "添加失败", "", "");
        }
    }

    @Override
    public Result batch(List<AccessLog> logs) {
        try {
            logs.forEach(log -> {
                log.setId(UUID.randomUUID().toString());
                if (StringUtils.hasText(log.getDetail())) {
                    log.setDetail(log.getDetail().replaceAll(REG_EXP, ""));
                }
                if (StringUtils.hasText(log.getResult())) {
                    log.setResult(log.getResult().replaceAll(REG_EXP, ""));
                }
            });
            this.logRepository.batch(logs);
            log.info("审计日志推送开关：" + enable);
            if (enable) {
                for (AccessLog accessLog : logs) {
                    log.info("开始推送审计日志到KAFKA，请求ID为：" + accessLog.getRequestId());
                    pushKafka(accessLog);
                }
            }
            return new Result(true, "success");
        } catch (Exception e) {
            log.error("batch save log error!", e);
            return new Result(false, e.getMessage());
        }
    }

    private List<Map<String, Object>> createNodes(String type, BoCloudService... services) {
        List<Map<String, Object>> nodes = Lists.newArrayList();
        for (BoCloudService service : services) {
            Map<String, Object> node = MapTools.simpleMap("type", type);
            node.put("name", service.getName());
            nodes.add(node);
        }
        return nodes;
    }

    @Override
    public GeneralResult<StringBuilder> archive(String startTime, String endTime, boolean deleted,
                                                RequestContext context, HttpServletRequest request, HttpServletResponse response) {
        try {
            List<Map<String, Object>> list = logRepository.list(startTime, endTime);
            StringBuilder builder = new StringBuilder();
            Optional.ofNullable(list).orElse(new ArrayList<>())
                    .forEach(log -> builder.append(log.values()).append("\n"));
            if (builder.length() > 0) {
                builder.insert(0, "请求对象  操作  请求IP  响应IP  请求时间  服务名称  请求耗时(ms)  操作者  参数详情\n");
            }
            LogArchiveRecord record = new LogArchiveRecord();
            User result = userRepository.query(context.getTarget());
            if (result != null) {
                record.setUserName(result.getName());
            }
            record.setUserId(context.getTarget());
            record.setGmtArchive(new Date());
            if (builder.length() > 0) {
                record.setResult("归档成功");
            } else {
                record.setResult("归档失败,此时间段无日志");
            }
            if (deleted) {
                record.setSyncDelete(Boolean.TRUE);
                logRepository.delete(startTime, endTime);
            } else {
                record.setSyncDelete(Boolean.FALSE);
            }
            logArchiveRecordRepository.save(record);
            byte[] bytes = builder.toString().getBytes();
            try {
                // 定义输出流
                response.setContentType("text/plain");
                response.setCharacterEncoding("utf-8");
                String filename = "审计日志.txt";
                final String userAgent = request.getHeader("USER-AGENT");
                if (userAgent.contains("Mozilla")) {
                    filename = new String(filename.getBytes(), "ISO8859-1");
                }
                response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
                // 定义输出流
                OutputStream outputStream = response.getOutputStream();
                outputStream.write(bytes);
                outputStream.close();
            } catch (Exception e) {
                log.error("Log archive error", e);
            }
            return new GeneralResult<>(true, builder, "日志归档成功");
        } catch (Exception e) {
            log.error("Archiving log has failed!", e);
            return new GeneralResult<>(false, "日志归档失败");
        }
    }

    private void pushKafka(AccessLog accessLog) {
        LogModel logModel = new LogModel();
        Map<Long, String> userMap = userRepository.listAll().stream().collect(Collectors.toMap(User::getId, User::getName));
        this.convertLog(accessLog, logModel, userMap);
        try {
            kafkaTemplate.send(topic, logModel).get(10, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("推送请求ID为{}审计日志数据失败：", accessLog.getRequestId(), e);
        }
    }

    private void convertLog(AccessLog accessLog, LogModel kafkaLog, Map<Long, String> userMap) {
        if (userMap.containsKey(accessLog.getUserId())) {
            kafkaLog.setSubjectName(userMap.get(accessLog.getUserId()));
        }
        kafkaLog.setSystemNumber(systemNumber);
        kafkaLog.setSystemNameCn(systemNameCn);
        kafkaLog.setSystemAddress(host);
        kafkaLog.setMessage(accessLog.getAction());
        kafkaLog.setSubjectAddress(accessLog.getRequestIp());
        kafkaLog.setStartTime(simpleFormat.format(accessLog.getGmtCreate()));
        String outcome = "0";
        try {
            GeneralResult result = JSONObject.parseObject(accessLog.getResult(), GeneralResult.class);
            if (result == null || result.isFailed()) {
                outcome = "1";
            }
        } catch (Exception e) {
            log.error("解析日志结果异常", e);
            outcome = "1";
        }
        kafkaLog.setOutcome(outcome);


    }
}
