package com.bocloud.sms.service.ldap;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ldap.filter.AndFilter;
import org.springframework.ldap.filter.EqualsFilter;
import org.springframework.util.StringUtils;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class LdapRepository {


    private final LdapConnector connector;

    public LdapRepository(LdapConnector connector) {
        super();
        this.connector = connector;
    }

    private DirContext getContext() {
        return this.connector.getConnector();
    }

    private String getBaseDn() {
        return this.connector.getBaseDn();
    }

    public JSONObject listAll(String dn) {
        List<JSONObject> depts = listDept(dn);
        List<JSONObject> managers = listUser(dn);
        if (ListTool.isEmpty(managers)) {
            managers = Lists.newArrayList();
        }
        if (ListTool.isEmpty(depts)) {
            depts = Lists.newArrayList();
        } else {
            union(depts, dn, managers);
        }
        JSONObject object = new JSONObject();
        object.put("depts", depts);
        object.put("managers", managers);
        return object;
    }

    public static void main(String[] args) {
        /*
         * LdapConnector connector = new LdapConnector("ldap://10.20.2.200:389/",
         * "cn=root,dc=domain,dc=com", "onceas", "dc=domain,dc=com");
         */
        LdapConnector connector = new LdapConnector("ldap://192.168.1.90:389/", "administrator@horizon.bocloud.com.cn",
                "P@ssw0rd", "dc=horizon,dc=bocloud,dc=com,dc=cn");
        LdapRepository repository = new LdapRepository(connector);
        connector.start();
        System.out.println(repository.listAll(""));
    }

    private void union(List<JSONObject> depts, String dn, List<JSONObject> users) {
        for (JSONObject dept : depts) {
            String deptDn = "ou=" + dept.getString("name");
            if (StringUtils.hasText(dn)) {
                deptDn += "," + dn;
            }
            List<JSONObject> managers = listUser(deptDn);
            if (!ListTool.isEmpty(managers)) {
                dept.put("managers", managers);
                users.addAll(managers);
            }
            List<JSONObject> children = JSONObject.parseArray(dept.getString("childs"), JSONObject.class);
            if (!ListTool.isEmpty(children)) {
                union(children, deptDn, users);
                dept.put("childs", children);
            }
        }
    }

    private List<JSONObject> listUser(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.ONELEVEL_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalPerson"));
        try {
            String base = !StringUtils.hasText(dn) ? this.getBaseDn() : dn + "," + this.getBaseDn();
            NamingEnumeration<SearchResult> results = this.getContext().search(base, filter.encode(), sc);
            List<JSONObject> list = new ArrayList<>();
            while (results.hasMoreElements()) {
                JSONObject manager = new JSONObject();
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("cn".equals(attribute.getID())) {
                            manager.put("name", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            manager.put("remark", attribute.get());
                        } else if ("uid".equals(attribute.getID())) {
                            manager.put("account", attribute.get());
                        } else if ("password".equals(attribute.getID())) {
                            manager.put("password", attribute.get());
                        } else if ("department".equals(attribute.getID())) {
                            manager.put("departName", attribute.get());
                        } else if ("userPrincipalName".equals(attribute.getID())) {
                            manager.put("account", attribute.get());
                        } else if ("mobile".equals(attribute.getID())) {
                            manager.put("mobile", attribute.get());
                        }else if ("mail".equals(attribute.getID())) {
                            manager.put("email", attribute.get());
                        }
                    }
                }
                if (!manager.isEmpty()) {
                    if (!StringUtils.hasText(manager.getString("name"))) {
                        manager.put("account", manager.getString("account"));
                    }
                    list.add(manager);
                }
            }
            return list;
        } catch (NamingException e) {
            log.error("list ldap manager fail:", e);
            return null;
        }
    }

    private List<JSONObject> listDept(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.ONELEVEL_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalUnit"));
        try {
            String base = !StringUtils.hasText(dn) ? this.getBaseDn() : dn + "," + this.getBaseDn();
            NamingEnumeration<SearchResult> results = this.getContext().search(base, filter.encode(), sc);
            List<JSONObject> list = new ArrayList<>();
            while (results.hasMoreElements()) {
                JSONObject department = new JSONObject();
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("ou".equals(attribute.getID())) {
                            department.put("name", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            department.put("remark", attribute.get());
                        }
                    }
                }
                if (!department.isEmpty()) {
                    list.add(department);
                }
            }
            if (!list.isEmpty()) {
                for (JSONObject dept : list) {
                    String deptDn = "ou=" + dept.getString("name");
                    if (StringUtils.hasText(dn)) {
                        deptDn += "," + dn;
                    }
                    List<JSONObject> children = listDept(deptDn);
                    if (!ListTool.isEmpty(children)) {
                        dept.put("childs", children);
                    }
                }
            }
            return list;
        } catch (Exception e) {
            log.error("list ldap dept fail:", e);
            return null;
        }

    }

}
