package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.megatron.framework.registry.RegistryClient;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.Result;
import com.megatron.common.utils.JSONTools;
import okhttp3.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author lixin
 * 微信通知
 */
@Component("weChatWorkSender")
public class WeChatWorkSender {
    private static final Logger log = LoggerFactory.getLogger(WeChatWorkSender.class);

    private static final String PREFIX = "https://qyapi.weixin.qq.com";
    private static String accessToken = "";
    private static Long expiresTime = 0L;
    private final OkHttpClient okHttpClient = new OkHttpClient();
    private final RegistryClient client;
    private String corpId = "";
    private String corpSecret = "";
    private String agentId = "";

    @Autowired
    public WeChatWorkSender(final RegistryClient client) {
        this.client = client;
    }

    /**
     * 向企业微信用户发送消息
     *
     * @param userIdString 接收成员ID,多成员用‘|’隔开，例如："UserID1|UserID2|UserID3"
     * @param content
     * @return errorCode
     */
    public Result pushMessage(String userIdString, String content) {
        Boolean flag = false;
        try {
            if (!StringUtils.hasText(accessToken)) {
                //首次发送请求
                setConfig();
                flag = true;
            }
            Assert.hasText(userIdString, "发送企业微信消息：userIdString不能为空");
            Assert.hasText(content, "发送企业微信消息：要发送的内容不能为空");

            int errorCode = pushMessage(userIdString, content, getOrFlushAccessToken(flag));
            if (errorCode == 40014 || errorCode == 42001) {
                //token不合法/已过期，强制刷新token
                accessToken = getOrFlushAccessToken(true);
                errorCode = pushMessage(userIdString, content, accessToken);
            }
            if (errorCode == 0) {
                return new Result(true, "");
            }
        } catch (Exception e) {
            log.error("send wechat message error:", e);
        }
        return new Result(false, "");
    }

    /**
     * 获取配置信息，设置配置信息
     *
     * @throws Exception
     */
    private void setConfig() throws Exception {
        String corpIdNow;
        String corpSecretNow;
        String agentIdNow;
        String data = this.client.getSystem();
        JSONObject object = JSONTools.isJSONObj(data);
        if (null != object) {
            corpIdNow = object.getString("WeChatWorkCorpId");
            corpSecretNow = object.getString("WeChatWorkCorpSecret");
            agentIdNow = object.getString("WeChatWorkAgentId");
            if (!StringUtils.hasText(corpIdNow)) {
                throw new Exception("get system config--WeChatWorkCorpId is null!");
            }
            if (!StringUtils.hasText(corpSecretNow)) {
                throw new Exception("get system config--WeChatWorkCorpSecret is null!");
            }
            if (!StringUtils.hasText(agentIdNow)) {
                throw new Exception("get system config--WeChatWorkAgentId is null!");
            }
        } else {
            throw new Exception("get system config is null");
        }
        log.info("set wechat config success!");
        this.corpSecret = new DESEncryptor().decrypt(corpSecretNow, "BocloudCMP");
        this.corpId = corpIdNow;
        this.agentId = agentIdNow;
    }

    private int pushMessage(String userIdString, String content, String accessTokenNow) {
        String url = PREFIX + "/cgi-bin/message/send?access_token=" + accessTokenNow;
        Map<String, Object> paramsMap = new HashMap<>();
        //指定消息成员
        paramsMap.put("touser", userIdString);
        //指定消息类型
        paramsMap.put("msgtype", "text");
        //应用id
        paramsMap.put("agentid", agentId);
        Map<String, Object> textMap = new HashMap<>();
        textMap.put("content", content);
        paramsMap.put("text", textMap);

        RequestBody requestBody = RequestBody.create(MediaType.parse(org.springframework.http.MediaType.APPLICATION_JSON_UTF8_VALUE), JSON.toJSONBytes(paramsMap));
        Request request = new Request.Builder()
                .header("Content-Type", "application/json")
                .post(requestBody)
                .url(url)
                .build();
        int errCode = -1;
        try {
            String result = null;
            Call call = okHttpClient.newCall(request);
            Response resp = call.execute();
            if (resp.body() != null) {
                result = resp.body().string();
            }
            JSONObject jsonObject = JSONObject.parseObject(result);
            errCode = jsonObject.getInteger("errcode");
            if (errCode != 0) {
                log.error("send wechat message failed, userIdString:{} ,content:{} ,result:{} ", userIdString, content, result);
            } else {
                log.info("send wechat message success, userIdString:{} ,content:{} ", userIdString, content);
            }
        } catch (Exception e) {
            log.error("send wechat message failed" + ",userIdString:" + userIdString + ",content:" + content, e);
        }
        return errCode;
    }


    /**
     * 获取/刷新token
     *
     * @param flushNow
     * @return
     * @throws Exception
     */
    private synchronized String getOrFlushAccessToken(boolean flushNow) throws Exception {
        if (System.currentTimeMillis() >= expiresTime || flushNow) {
            String result = getToken();
            JSONObject jsonObject = JSONObject.parseObject(result);
            int errCode = jsonObject.getInteger("errcode");
            if (errCode == 0) {
                accessToken = jsonObject.getString("access_token");
                int expiresIn = jsonObject.getInteger("expires_in");
                //提前5分钟过期
                expiresTime = (expiresIn - 300) * 1000 + System.currentTimeMillis();
                log.info("flush access token success,accessToken:{}", accessToken);
            } else {
                throw new Exception("flush access token error,result:" + result);
            }
        }
        return accessToken;
    }

    /**
     * 调用获取token api
     *
     * @return
     */
    public String getToken() throws Exception {
        this.setConfig();
        String url = PREFIX + "/cgi-bin/gettoken?corpid=" + corpId + "&corpsecret=" + corpSecret;
        Request request = new Request.Builder()
                .header("Content-Type", "application/json")
                .url(url)
                .build();
        String result = null;
        Call call = okHttpClient.newCall(request);
        try {
            Response resp = call.execute();
            if (resp.body() != null) {
                result = resp.body().string();
            }
        } catch (IOException e) {
            log.error("get token error,", e);
        }
        return result;
    }

}
